using System;
using System.Collections;

namespace Esempio1
{
	public class Contenitore : IEnumerable
	{
        //  Carattere iniziale della sequenza
		private readonly char _c;
        //  Numero di caratteri della sequenza
		private readonly int _length;

		public Contenitore(char c, int length)
		{
            if (length < 0)
                throw new ArgumentException("length < 0");
			_c = c;
			_length = length;
		}

		public int Length
		{
			get { return _length; }
		}

		public char this[int index]
		{
			get
			{
				if(index < 0 || index >= _length)
					throw new IndexOutOfRangeException();
				return (char) (_c + index);
			}
        }

        #region IEnumerable Members

        public IEnumerator GetEnumerator()
        {
            return new Enumeratore(this);
        }

        #endregion

        private class Enumeratore : IEnumerator
		{
			private readonly Contenitore _contenitore;
			private int _index;

			public Enumeratore(Contenitore contenitore)
			{
				_contenitore = contenitore;
				Reset();
			}

			#region IEnumerator Members

			public void Reset()
			{
				_index = -1;
			}

			public bool MoveNext()
			{
				if(_index >= _contenitore._length)
					throw new InvalidOperationException();
				_index++;
				return (_index < _contenitore._length);
			}

			public object Current
			{
				get
				{
					if(_index == -1 || _index >= _contenitore._length)
						throw new InvalidOperationException();
					return _contenitore[_index];
				}
			}

			#endregion
		}
    }
}
