﻿using System;
using System.Collections;
using System.Collections.Generic;

namespace Esempio1
{
    public class PowerGenerator : IEnumerable
    {
        private readonly int _number;
        private readonly int _exponent;

        private PowerGenerator(int number, int exponent)
        {
            _number = number;
            _exponent = exponent;
        }

        public static IEnumerable GetPowers1(int number, int exponent)
        {
            return new PowerGenerator(number, exponent);
        }

        #region IEnumerable Members

        IEnumerator IEnumerable.GetEnumerator()
        {
            return new PowerEnumerator(_number, _exponent);
        }

        #endregion

        #region List

        public static IEnumerable GetPowers2(int number, int exponent)
        {
            List<int> powers = new List<int>();
            for (int result = 1, counter = 1; counter <= exponent; counter++)
            {
                result = result * number;
                powers.Add(result);
            }
            return powers;
        }

        #endregion

        #region yield

        public static IEnumerable GetPowers3(int number, int exponent)
        {
            for (int result = 1, counter = 1; counter <= exponent; counter++)
            {
                result = result * number;
                yield return result;
            }
        }

        #endregion

        class PowerEnumerator : IEnumerator
        {
            private int _result;
            private int _counter;
            private readonly int _number;
            private readonly int _exponent;

            public PowerEnumerator(int number, int exponent)
            {
                _number = number;
                _exponent = exponent;
                Reset();
            }

            #region IEnumerator Members

            public void Reset()
            {
                _result = 1;
                _counter = 0;
            }

            public bool MoveNext()
            {
                if (_counter > _exponent)
                    throw new InvalidOperationException();
                _counter++;
                if (_counter > _exponent)
                    return false;
                _result = _result * _number;
                return true;
            }

            public object Current
            {
                get
                {
                    if (_counter == -1 || _counter > _exponent)
                        throw new InvalidOperationException();
                    return _result;
                }
            }

            #endregion
        }
    }
}
